clear all

program define Descfile
    syntax using/, book(string)
    
    qui use "`using'", clear
    
    if `"`: data label'"' != "" {
        file write `book' `"<h2>`:data label'</h2>"' _n
    }
    if "`: char _dta[note0]'" != "" {
        file write `book' "<h3>Notes:</h3>"_n"<ul>"_n
        forvalues i = 1/`: char _dta[note0]' {
            file write `book' `"<li>`: char _dta[note`i']'</li>"' _n
        }
        file write `book' "</ul>"_n
    }
end

program define Descvars
    syntax, book(string)
    
    file write `book' "<h3>Variable list</h3>"
    file write `book' "<ul>"_n
    foreach var of varlist * {
        file write `book' "<li>`var'"
        if `"`: var label `var''"' != "`var'" {
            file write `book' `": `: var label `var''"'
        }
        file write `book' "</li>"_n
        if "`: char `var'[note0]'" != "" {
            file write `book' "<ul>" _n
            forvalues i = 1/`: char `var'[note0]'{
                file write `book' `"<li>`: char `var'[note`i']'</li>"'
            }
            file write `book' "</ul>" _n
        }
    }
    file write `book' "</ul>"
end    

program define Descvalues
    syntax, book(string) file(string)
    
    local N = _N
    foreach var of varlist * {
        file write `book' `"<h4>`var'"'
        local varlab : var label `var'
        if `"`varlab'"' != "`var'" {
            file write `book' `": `varlab'"'
        }
        file write `book' "</h4>"_n
        tempvar freq
        contract `var', freq(`freq')
        
        local vallab : val label `var'    
        if "`vallab'" != "" {
            tempfile freqtable
            rename `var' value
            qui save `freqtable'
            qui uselabel `vallab'
            qui merge 1:1 value using `freqtable'
            file write `book' "<table>"_n
            file write `book' "<tr>"
            file write `book' "<th>value</th>"
            file write `book' "<th>label</th>"
            file write `book' "<th>frequency</th>"
            file write `book' "</tr>"_n
            forvalues i = 1/`=_N' {
                file write `book' "<tr>"
                file write `book' `"<td>`=value[`i']'</td>"'
                file write `book' `"<td>`=label[`i']'</td>"'
                file write `book' `"<td>`=`freq'[`i']'</td>"'
                file write `book' "</td>"_n
            }
            file write `book' "</table>"_n
        }
        else {                                                       //new
            if _N <= 10 {                                            //new
                file write `book' "<table>"_n                        //new
                file write `book' "<tr>"                             //new
                file write `book' "<th>value</th>"                   //new
                file write `book' "<th>frequency</th>"               //new
                file write `book' "</tr>"_n                          //new
                forvalues i = 1/`=_N' {                              //new
                    file write `book' "<tr>"                         //new
                    file write `book' `"<td>`=`var'[`i']'</td>"'     //new      
                    file write `book' `"<td>`=`freq'[`i']'</td>"'    //new
                    file write `book' "</td>"_n                      //new
                }                                                     //new
            }                                                        //new
            capture confirm numeric variable `var'                   //new
            else if !_rc {                                           //new
                file write `book' "<table>" _n                       //new
                qui count if !missing(`var')                         //new
                local distinct = r(N)                                //new
                qui sum `var' [fw=`freq'], detail                    //new
                file write `book' "<tr>"_n                           //new
                file write `book' `"<th>valid/missing obs.</th>"'    //new
                file write `book' `"<td>`r(N)'/`=`N'-`r(N)'' </td>"' //new
                file write `book' "</tr>"_n                          //new 
                file write `book' "<tr>"                             //new
                file write `book' "<th>distinct values</th>"         //new
                file write `book' "<td>`distinct'</td>"              //new
                file write `book' "</tr>"_n                          //new
                file write `book' "<tr>"                             //new
                file write `book' "<th>minimum</th>"                 //new
                file write `book' "<td>`r(min)'</td>"                //new
                file write `book' "</tr>"_n                          //new
                file write `book' "</table>" _n                      //new
            }                                                        //new
        }                                                            //new
        qui use `file', clear
    }

end

program define htmlcodebook
    version 14
    syntax using/ , SAVing(string) [replace title(string)]
    
    tempname book 
    file open `book' using `saving', write `replace'
    preserve
    
    file write `book' "<!DOCTYPE html>"_n"<html>"_n
    file write `book' "<style>"_n
    file write `book' "body {"_n
    file write `book' "width: 650px;"_n
    file write `book' "margin: auto;"_n
    file write `book' "}"_n
    file write `book' "</style>"_n
    file write `book' "<body>"_n
    
    if `"`title'"' != "" {
        file write `book' "<h1>`title'</h1>"_n
    }
    else {
        file write `book' "<h1>Codebook for `using'</h1>"_n
    }
    
    Descfile using "`using'", book(`book')
    Descvars, book(`book')
    Descvalues, book(`book') file(`using')    
    
    file write `book' "</body>"_n
    file write `book' "</html>"_n
    file close `book'
    di as txt "Output written to " `"{browse "`saving'"}"'    
    restore
end

cd h:\ss18\stata_l2
htmlcodebook using arc06.dta, saving(test.html) replace

